from pathlib import Path
from typing import NamedTuple, Callable
import logging

from .tasks import Task

logger = logging.getLogger(__name__)


class ConversionConfig(NamedTuple):
    input_dir: Path
    output_ext: str
    input_ext: str


def _get_converted_path(input_path: Path, config: ConversionConfig):
    """
    Return the path you would obtain if you moved the file in the input
    path to the output directory in the config and changed its extension.
    """

    relative_path = input_path.relative_to(config.input_dir)
    output_filename = str(input_path.stem) + f".{config.output_ext}"
    return relative_path.parent / output_filename


def get_unconverted_files(
    input_files: list[Path], config: ConversionConfig
) -> list[Path]:
    output_files = [_get_converted_path(f, config) for f in input_files]

    conversion_pairs = []
    for input_file, output_file in zip(input_files, output_files):
        if not output_file.exists():
            conversion_pairs.append((input_file, output_file))
    return conversion_pairs


def convert(config: ConversionConfig, conversion_func: Callable):

    logger.info("Converting files in %s", config.input_dir)
    logger.info("Writing output to: %s", config.output_dir)

    input_files = get_files_recursive(config.input_dir.resolve(), config.input_ext)
    output_files = get_uncoverted_files(input_files, config)

    tasks = _build_conversion_tasks(input_files, output_files, config, conversion_func)
    run_tasks(tasks)
