import argparse
import logging
from pathlib import Path

from jgutils import music

logger = logging.getLogger(__name__)


def cli_music_convert(args):

    config = music.CovertionConfig(
        args.input_dir.resolve(), args.output_dir.resolve(), "mp3", "flac"
    )
    music.convert(config)

def cli_music_metadata(args):

    collection = music.get_metadata(args.input_dir.resolve(), "flac")
    with open(args.output_path.resolve(), 'w') as f:
        f.write(collection.model_dump_json(indent=4))

def cli_music_refresh(args):

    collection = music.refresh(args.input_dir.resolve())

def main_cli():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(required=True)

    music_parser = subparsers.add_parser("music")
    music_subparsers = music_parser.add_subparsers(required=True)

    music_convert_parser = music_subparsers.add_parser("convert")
    music_convert_parser.add_argument(
        "--input_dir",
        type=Path,
        default=Path(),
        help="Directory with input files for conversion.",
    )
    music_convert_parser.add_argument(
        "--output_dir", type=Path, default=Path(), help="Directory for converted files"
    )
    music_convert_parser.set_defaults(func=cli_music_convert)
    
    music_md_parser = music_subparsers.add_parser("metadata")
    music_md_parser.add_argument(
        "--input_dir",
        type=Path,
        default=Path(),
        help="Directory with input music files.",
    )
    music_md_parser.add_argument(
        "--output_path",
        type=Path,
        default=Path() / "music_collection.json",
        help="Path to save collection to.",
    )
    music_md_parser.set_defaults(func=cli_music_metadata)

    music_refresh_parser = music_subparsers.add_parser("refresh")
    music_refresh_parser.add_argument(
        "--input_dir",
        type=Path,
        default=Path(),
        help="Directory with input music files.",
    )
    music_refresh_parser.set_defaults(func=cli_music_refresh)

    logging.basicConfig()
    logging.getLogger().setLevel(logging.INFO)

    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main_cli()
